package cn.jyjz.xiaoyao.framework.security.captcha.controller;

import cn.jyjz.xiaoyao.common.base.config.XiaoyaoConfig;
import cn.jyjz.xiaoyao.common.base.constant.RequestParameterConstans;
import cn.jyjz.xiaoyao.common.base.util.UUIDUtils;
import cn.jyjz.xiaoyao.common.base.vo.ResultVo;
import cn.jyjz.xiaoyao.common.base.vo.ResultVoUtil;
import cn.jyjz.xiaoyao.common.redis.cache.ICacheManager;
import cn.jyjz.xiaoyao.common.redis.cache.constants.CacheConstants;
import cn.jyjz.xiaoyao.common.redis.config.CacheProperties;
import com.google.code.kaptcha.Producer;
import java.awt.image.BufferedImage;
import java.io.IOException;
import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.ModelAndView;

/**
 * Created by zhangyong
 */
@Api(tags="验证码")
@RestController
@RequestMapping("/captcha")
public class CaptchaController {

    private Logger logger = LoggerFactory.getLogger(CaptchaController.class);

    @Resource(name = "captchaProducer")
    private Producer captchaProducer;

    @Resource(name = "captchaProducerMath")
    private Producer captchaProducerMath;

    @Autowired
    private ICacheManager iCacheManager;

    @Autowired private XiaoyaoConfig xiaoyaoConfig;

    @Autowired
    private CacheProperties cacheProperties;


    /**
     * 验证码生成的token
     * 通过这个token获得验证码
     */
    @ApiOperation(value="获得验证码主键", notes="获得验证码主键")
    @GetMapping(value = "/captchatoken")
    public ResultVo getKaptchaToken(HttpServletRequest request, HttpServletResponse response)
    {
        //验证码类型，math表示计算图片，char表示图片
        String type = request.getParameter(RequestParameterConstans.CACHE_NAME);
        String capText = null;
        if (RequestParameterConstans.CACHE_MATH.equals(type))
        {
            capText = captchaProducerMath.createText();
        }
        else if (RequestParameterConstans.CACHE_CHAR.equals(type))
        {
            capText = captchaProducer.createText();
        }else{
            return ResultVoUtil.error();
        }

        //组装验证码内容：math_
        capText = type + cacheProperties.getCacheNamePrix() +capText;
        //生成随机码，
        String token = UUIDUtils.getUUID();

        iCacheManager.put(CacheConstants.CACHE_CODE ,token,capText,CacheConstants.CACHE_TIME);

        return ResultVoUtil.success(token);
    }

    /**
     * 验证码生成
     */
    @ApiOperation(value="获得验证码", notes="获得验证码")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path",name = "codeToken", value =  "验证码主键", required = true)
    })
    @GetMapping(value = "/captchaImage")
    public ModelAndView getKaptchaImage(HttpServletRequest request, HttpServletResponse response)
    {
        ServletOutputStream out = null;
        try
        {
            response.setDateHeader("Expires", 0);
            response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
            response.addHeader("Cache-Control", "post-check=0, pre-check=0");
            response.setHeader("Pragma", "no-cache");
            response.setContentType("image/jpeg");


            //获得缓存的关键字token
            String token = request.getParameter(RequestParameterConstans.CACHE_CODE_PARAMETER);

            String capStr = null;
            BufferedImage bi = null;

            String capText = iCacheManager.get(CacheConstants.CACHE_CODE ,token);

            if(StringUtils.isBlank(capText)){
                return null;
            }
            //验证码类型，math表示计算图片，char表示图片
            String type = capText.substring(0,capText.indexOf(cacheProperties.getCacheNamePrix()));

            if (RequestParameterConstans.CACHE_MATH.equals(type))
            {
                capStr = capText.substring(capText.lastIndexOf(cacheProperties.getCacheNamePrix())+1, capText.lastIndexOf("@"));

                bi = captchaProducerMath.createImage(capStr);
            }
            else if (RequestParameterConstans.CACHE_CHAR.equals(type))
            {
                capStr = capText.substring(capText.lastIndexOf(cacheProperties.getCacheNamePrix())+1);
                bi = captchaProducer.createImage(capStr);
            }

            out = response.getOutputStream();
            ImageIO.write(bi, "jpg", out);
            out.flush();

        }
        catch (Exception e)
        {
            logger.error("CaptchaController生成验证码错误:{}",e);
        }
        finally
        {
            try
            {
                if (out != null)
                {
                    out.close();
                }
            }
            catch (IOException e)
            {
                logger.error("CaptchaController生成验证码错误:{}",e);
            }
        }
        return null;
    }
}
