package cn.jyjz.xiaoyao.framework.security.model;

import cn.jyjz.xiaoyao.common.base.vo.User;
import cn.jyjz.xiaoyao.common.base.vo.UserToken;
import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.util.Assert;

/**
 * Created by zhangyong on 2021/3/8.
 */
public class AdminUserModel implements UserDetails {
    
    //loginname + uuid用于缓存的key使用,与UserToken的id保持一致
    private String id;
    
    private String username;
    
    private String password;
    
    private Set<GrantedAuthority> authorities;
    
    private User user;
    
    private UserToken userToken;
    
    //存放flowableuser
    private Object flowableUser;
    
    //浏览器识别信息
    private String browerUUID;
    //企业编码
    String agentcode;
    //账户是否可用
    boolean enabled;
    
    //账号是否没过期
    boolean accountNonExpired;
    //账号是否没被锁定
    boolean accountNonLocked;
    //密码是否没过期
    boolean credentialsNonExpired;
    
    public AdminUserModel(String id, String username, String password, boolean enabled, boolean accountNonExpired,
                          boolean credentialsNonExpired, boolean accountNonLocked,
                          Collection<? extends GrantedAuthority> authorities) {
        
        this.id = id;
        this.username = username;
        this.password = password;
        this.enabled = enabled;
        this.accountNonExpired = accountNonExpired;
        this.credentialsNonExpired = credentialsNonExpired;
        this.accountNonLocked = accountNonLocked;
        this.authorities = Collections.unmodifiableSet(sortAuthorities(authorities));
        
    }
    
    private static SortedSet<GrantedAuthority> sortAuthorities(Collection<? extends GrantedAuthority> authorities) {
        Assert.notNull(authorities, "Cannot pass a null GrantedAuthority collection");
        TreeSet sortedAuthorities = new TreeSet(new AuthorityComparator());
        Iterator var2 = authorities.iterator();
        
        while (var2.hasNext()) {
            GrantedAuthority grantedAuthority = (GrantedAuthority) var2.next();
            Assert.notNull(grantedAuthority, "GrantedAuthority list cannot contain any null elements");
            sortedAuthorities.add(grantedAuthority);
        }
        
        return sortedAuthorities;
    }
    
    private static class AuthorityComparator implements Comparator<GrantedAuthority>, Serializable {
        private static final long serialVersionUID = 540L;
        
        private AuthorityComparator() {
        }
        
        @Override
        public int compare(GrantedAuthority g1, GrantedAuthority g2) {
            return g2.getAuthority() == null ? -1 : (g1.getAuthority() == null ? 1 : g1.getAuthority().compareTo(g2.getAuthority()));
        }
    }
    
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }
    
    public void setAccountNonExpired(boolean accountNonExpired) {
        this.accountNonExpired = accountNonExpired;
    }
    
    public void setAccountNonLocked(boolean accountNonLocked) {
        this.accountNonLocked = accountNonLocked;
    }
    
    public void setCredentialsNonExpired(boolean credentialsNonExpired) {
        this.credentialsNonExpired = credentialsNonExpired;
    }
    
    public void setAuthorities(Set<GrantedAuthority> authorities) {
        this.authorities = authorities;
    }
    
    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return this.authorities;
    }
    
    
    public String getId() {
        return id;
    }
    
    public void setId(String id) {
        this.id = id;
    }
    
    public void setUsername(String username) {
        this.username = username;
    }
    
    public void setPassword(String password) {
        this.password = password;
    }
    
    @Override
    public String getPassword() {
        return this.password;
    }
    
    @Override
    public String getUsername() {
        return this.username;
    }
    
    @Override
    public boolean isAccountNonExpired() {
        return this.accountNonExpired;
    }
    
    @Override
    public boolean isAccountNonLocked() {
        return this.accountNonLocked;
    }
    
    @Override
    public boolean isCredentialsNonExpired() {
        return this.credentialsNonExpired;
    }
    
    @Override
    public boolean isEnabled() {
        return this.enabled;
    }
    
    public User getUser() {
        return user;
    }
    
    public void setUser(User user) {
        this.user = user;
    }
    
    public UserToken getUserToken() {
        return userToken;
    }
    
    public void setUserToken(UserToken userToken) {
        this.userToken = userToken;
    }
    
    public Object getFlowableUser() {
        return flowableUser;
    }
    
    public void setFlowableUser(Object flowableUser) {
        this.flowableUser = flowableUser;
    }
    
    public String getBrowerUUID() {
        return browerUUID;
    }
    
    public void setBrowerUUID(String browerUUID) {
        this.browerUUID = browerUUID;
    }
    
    public String getAgentcode() {
        return agentcode;
    }
    
    public void setAgentcode(String agentcode) {
        this.agentcode = agentcode;
    }
}
